@@ -3,11 +3,13 @@ package ExtUtils::CBuilder::Base;
 use strict;
 use File::Spec;
 use File::Basename;
+use Cwd ();
 use Config;
 use Text::ParseWords;
+use IO::File;
 
 use vars qw($VERSION);
-$VERSION = '0.12';
+$VERSION = '0.24_01';
 
 sub new {
   my $class = shift;
@@ -117,10 +119,8 @@ sub have_compiler {
   
   my $tmpfile = File::Spec->catfile(File::Spec->tmpdir, 'compilet.c');
   {
-    local *FH;
-    open FH, "> $tmpfile" or die "Can't create $tmpfile: $!";
-    print FH "int boot_compilet() { return 1; }\n";
-    close FH;
+    my $FH = IO::File->new("> $tmpfile") or die "Can't create $tmpfile: $!";
+    print $FH "int boot_compilet() { return 1; }\n";
   }
 
   my ($obj_file, @lib_files);
@@ -244,7 +244,7 @@ sub perl_src {
   my $dir   = File::Spec->curdir;
 
   # Try up to 5 levels upwards
-  for (0..5) {
+  for (0..10) {
     if (
 	-f File::Spec->catfile($dir,"config_h.SH")
 	&&
@@ -252,7 +252,7 @@ sub perl_src {
 	&&
 	-f File::Spec->catfile($dir,"lib","Exporter.pm")
        ) {
-      return $dir;
+      return Cwd::realpath( $dir );
     }
 
     $dir = File::Spec->catdir($dir, $Updir);
@@ -271,6 +271,7 @@ sub perl_inc {
 
 sub DESTROY {
   my $self = shift;
+  local($., $@, $!, $^E, $?);
   $self->cleanup();
 }
 
